@@ -15,17 +15,21 @@ use merkle_cbt::{merkle_tree::Merge, MerkleProof, MerkleTree, CBMT};
 
 /// Possible errors in the crate
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub enum Error<K> {
+pub enum Error<'a, K> {
+    /// Empty tree
+    EmptyTree,
     /// The proof is invalid
     InvalidProof,
     /// Key already included in tree
-    KeyIncluded(K),
+    KeyIncluded(&'a K),
     /// Key not coverted in proof
-    KeyUnknown(K),
+    KeyUnknown(&'a K),
 }
 
+/// Type alias to `[u8; 32]`
 pub type H256 = [u8; 32];
 
+// TODO: add example blake2b hasher
 /// Trait for customize hash function
 pub trait Hasher {
     /// Update data into the hasher
@@ -163,18 +167,19 @@ where
         &self.raw_proof
     }
 
-    /// Verify the `keys` are all not in tree, `None` means the `leaves` is not in the tree.
+    // TODO: add verify exclusion example
+    /// Verify the `excluded_keys` are all not in tree.
     ///
     ///  * `Ok(())`                    => All keys are not in tree
     ///  * `Err(Error::InvalidProof)`  => The proof don't match the root
     ///  * `Err(Error::KeyIncluded(K))`=> Some keys are in tree
     ///  * `Err(Error::KeyUnknown(K))` => The proof is ok, but some keys not coverted in the range
-    pub fn verify_exclusion<K, V, H>(
+    pub fn verify_exclusion<'a, K, V, H>(
         &self,
         root: &H256,
         range_leaves: &[RangeLeaf<K, V, H>],
-        keys: &[K],
-    ) -> Result<(), Error<K>>
+        excluded_keys: &'a [K],
+    ) -> Result<(), Error<'a, K>>
     where
         K: Ord + AsRef<[u8]> + Clone,
         V: AsRef<[u8]> + Default + Clone,
@@ -182,11 +187,11 @@ where
     {
         let leaf_hashes: Vec<H256> = range_leaves.iter().map(RangeLeaf::hash).collect();
         if self.raw_proof.verify(root, &leaf_hashes) {
-            for key in keys {
+            for key in excluded_keys {
                 let mut excluded = false;
                 for range_leaf in range_leaves {
                     if range_leaf.match_either_key(key) {
-                        return Err(Error::KeyIncluded(key.clone()));
+                        return Err(Error::KeyIncluded(key));
                     }
                     if range_leaf.match_range(key) {
                         excluded = true;
@@ -194,7 +199,7 @@ where
                     }
                 }
                 if !excluded {
-                    return Err(Error::KeyUnknown(key.clone()));
+                    return Err(Error::KeyUnknown(key));
                 }
             }
             Ok(())
@@ -215,6 +220,7 @@ impl<M> From<ExclusionMerkleProof<M>> for MerkleProof<H256, M> {
     }
 }
 
+// TODO: add example to build proof
 /// A helper struct to build data structure for verifing the exclusion of keys
 ///
 ///  * range leaves
@@ -236,56 +242,98 @@ where
     H: Hasher + Default,
     M: Merge<Item = H256>,
 {
+    // TODO: explain how range leaf built
     /// Build range leaves by raw leaves
-    pub fn build_range_leaves(mut raw_leaves: Vec<Leaf<K, V>>) -> Vec<RangeLeaf<K, V, H>> {
+    pub fn build_range_leaves(raw_leaves: &[Leaf<K, V>]) -> Vec<RangeLeaf<K, V, H>> {
         if raw_leaves.is_empty() {
             return Vec::new();
         }
+        let mut raw_leaves: Vec<&Leaf<K, V>> = raw_leaves.iter().collect();
         raw_leaves.sort_unstable_by(|a, b| a.key.cmp(&b.key));
         let mut range_leaves: Vec<_> = Vec::with_capacity(raw_leaves.len());
         for window in raw_leaves.windows(2) {
-            range_leaves.push(window[0].to_range(&window[1]));
+            range_leaves.push(window[0].to_range(window[1]));
         }
-        range_leaves.push(raw_leaves[raw_leaves.len() - 1].to_range(&raw_leaves[0]));
+        range_leaves.push(raw_leaves[raw_leaves.len() - 1].to_range(raw_leaves[0]));
         range_leaves
     }
 
     /// Build merkle root
-    pub fn build_merkle_root(raw_leaves: &[Leaf<K, V>]) -> H256 {
+    pub fn compute_root(raw_leaves: &[Leaf<K, V>]) -> H256 {
         if raw_leaves.is_empty() {
             return Default::default();
         }
-        let range_leaves = Self::build_range_leaves(raw_leaves.to_vec());
+        let range_leaves = Self::build_range_leaves(raw_leaves);
         let range_leaf_hashes: Vec<_> = range_leaves.iter().map(RangeLeaf::hash).collect();
         CBMT::<H256, M>::build_merkle_root(&range_leaf_hashes)
     }
 
     /// Build merkle tree
-    pub fn build_merkle_tree(raw_leaves: Vec<Leaf<K, V>>) -> MerkleTree<H256, M> {
-        let range_leaves = Self::build_range_leaves(raw_leaves.to_vec());
+    pub fn build_tree(raw_leaves: &[Leaf<K, V>]) -> MerkleTree<H256, M> {
+        let range_leaves = Self::build_range_leaves(raw_leaves);
         let range_leaf_hashes: Vec<_> = range_leaves.iter().map(RangeLeaf::hash).collect();
         CBMT::<H256, M>::build_merkle_tree(&range_leaf_hashes)
     }
 
+    /// Build proof by `excluded_keys`
+    pub fn build_proof<'a>(
+        raw_leaves: &[Leaf<K, V>],
+        excluded_keys: &'a [K],
+    ) -> Result<(ExclusionMerkleProof<M>, Vec<RangeLeaf<K, V, H>>), Error<'a, K>> {
+        if raw_leaves.is_empty() {
+            return Err(Error::EmptyTree);
+        }
+        let mut excluded_keys: Vec<&'a K> = excluded_keys.iter().collect();
+        excluded_keys.sort_unstable();
+        // Range leaves are sorted too
+        let range_leaves = Self::build_range_leaves(raw_leaves);
+        let mut excluded_index = 0;
+        let mut indices = Vec::new();
+        let mut required_range_leaves = Vec::new();
+        let match_last_range = excluded_keys[0] < &range_leaves[0].key;
+        for (idx, range_leaf) in range_leaves.into_iter().enumerate() {
+            let mut match_current_range = false;
+            while excluded_index < excluded_keys.len() {
+                let key = excluded_keys[excluded_index];
+                if range_leaf.match_range(key) {
+                    match_current_range = true;
+                } else if range_leaf.match_either_key(key) {
+                    return Err(Error::KeyIncluded(key));
+                } else if key > &range_leaf.next_key {
+                    break;
+                }
+                excluded_index += 1;
+            }
+            if match_current_range || ((idx == raw_leaves.len() - 1) && match_last_range) {
+                indices.push(idx as u32);
+                required_range_leaves.push(range_leaf);
+            }
+        }
+        Self::build_proof_by_indices(raw_leaves, &indices)
+            .map(|proof| (proof, required_range_leaves))
+    }
+
+    // TODO: add build merkle proof example
     /// Build merkle proof
-    pub fn build_merkle_proof(
+    pub fn build_proof_by_indices<'a>(
         raw_leaves: &[Leaf<K, V>],
         indices: &[u32],
-    ) -> Option<ExclusionMerkleProof<M>> {
-        Self::build_merkle_tree(raw_leaves.to_vec())
+    ) -> Result<ExclusionMerkleProof<M>, Error<'a, K>> {
+        Self::build_tree(raw_leaves)
             .build_proof(indices)
             .map(Into::into)
+            .ok_or(Error::EmptyTree)
     }
 }
 
-/// Simple empty Leaf value
-pub type SimpleValue = [u8; 0];
-/// Simple Leaf binded to SimpleValue
-pub type SimpleLeaf<K> = Leaf<K, SimpleValue>;
-/// Simple RangeLeaf binded to SimpleValue
-pub type SimpleRangeLeaf<K, H> = RangeLeaf<K, SimpleValue, H>;
-/// Simple ExclusionCBMT binded to SimpleValue
-pub type SimpleExclusionCBMT<K, H, M> = ExclusionCBMT<K, SimpleValue, H, M>;
+/// Empty Leaf value
+pub type EmptyValue = [u8; 0];
+/// Simple Leaf binded to EmptyValue
+pub type SimpleLeaf<K> = Leaf<K, EmptyValue>;
+/// Simple RangeLeaf binded to EmptyValue
+pub type SimpleRangeLeaf<K, H> = RangeLeaf<K, EmptyValue, H>;
+/// Simple ExclusionCBMT binded to EmptyValue
+pub type SimpleExclusionCBMT<K, H, M> = ExclusionCBMT<K, EmptyValue, H, M>;
 
 #[cfg(test)]
 mod tests {
@@ -328,24 +376,48 @@ mod tests {
     type StrKey = &'static str;
     type StrLeaf = SimpleLeaf<StrKey>;
     type StrRangeLeaf = SimpleRangeLeaf<StrKey, Blake2bHasher>;
+    // A helper to compute root and build proof
     type StrExCBMT = SimpleExclusionCBMT<StrKey, Blake2bHasher, MergeBlake2bH256>;
 
     #[test]
     fn test_simple() {
+        // Can be seen as a black list
         let all_leaves: Vec<StrLeaf> = vec!["b", "e", "g", "x"]
             .into_iter()
             .map(StrLeaf::new_with_key)
             .collect();
-        let all_range_leaves = StrExCBMT::build_range_leaves(all_leaves.clone());
+        let root = StrExCBMT::compute_root(&all_leaves);
+        // The keys not in the black list
+        let excluded_keys = vec!["f", "y", "z", "a"];
+        let (proof, range_leaves) = StrExCBMT::build_proof(&all_leaves, &excluded_keys).unwrap();
+        assert_eq!(
+            range_leaves
+                .iter()
+                .map(|l| (*l.key(), *l.next_key()))
+                .collect::<Vec<_>>(),
+            vec![("e", "g"), ("x", "b")]
+        );
+        assert!(proof
+            .verify_exclusion(&root, &range_leaves, &excluded_keys)
+            .is_ok());
+    }
+
+    #[test]
+    fn test_build_by_indices() {
+        let all_leaves: Vec<StrLeaf> = vec!["b", "e", "g", "x"]
+            .into_iter()
+            .map(StrLeaf::new_with_key)
+            .collect();
+        let all_range_leaves = StrExCBMT::build_range_leaves(&all_leaves);
         // ["e", "x"] => [("e", "g"), ("x", "b")]
         let indices: Vec<u32> = vec![1, 3];
         let range_leaves: Vec<StrRangeLeaf> = indices
             .iter()
             .map(|index| all_range_leaves[*index as usize].clone())
             .collect();
-        let root = StrExCBMT::build_merkle_root(&all_leaves);
+        let root = StrExCBMT::compute_root(&all_leaves);
         let proof: ExclusionMerkleProof<MergeBlake2bH256> =
-            StrExCBMT::build_merkle_proof(&all_leaves, &indices).unwrap();
+            StrExCBMT::build_proof_by_indices(&all_leaves, &indices).unwrap();
 
         assert_eq!(
             range_leaves
@@ -389,21 +461,21 @@ mod tests {
         let excluded_keys: Vec<StrKey> = vec!["e"];
         assert_eq!(
             proof.verify_exclusion(&root, &range_leaves, &excluded_keys),
-            Err(Error::KeyIncluded("e"))
+            Err(Error::KeyIncluded(&"e"))
         );
 
         // "e","x" are in included keys
         let excluded_keys: Vec<StrKey> = vec!["e", "f", "x"];
         assert_eq!(
             proof.verify_exclusion(&root, &range_leaves, &excluded_keys),
-            Err(Error::KeyIncluded("e"))
+            Err(Error::KeyIncluded(&"e"))
         );
 
         // "c" is not in included keys, but the proof can not verify it
         let excluded_keys: Vec<StrKey> = vec!["c"];
         assert_eq!(
             proof.verify_exclusion(&root, &range_leaves, &excluded_keys),
-            Err(Error::KeyUnknown("c"))
+            Err(Error::KeyUnknown(&"c"))
         );
     }
 }
